@@ -1,15 +1,15 @@
+import gzip
 import os
 from collections import defaultdict
-from typing import Dict, List
+from typing import Any, Dict, List
 
 import numpy as np
-import orjson
 import pandas as pd
 from numba import types
 from numba.typed import Dict as TypedDict
 from numba.typed import List as TypedList
 
-from ..io import download
+from ..io import download, load_json, load_lz4, save_json, save_lz4
 from .common import (
     add_and_sort,
     create_and_sort,
@@ -17,11 +17,13 @@ from .common import (
     sort_dict_of_dict_by_value,
     to_typed_list,
 )
+from .generic import create_empty_results_dict
+from .qrels import Qrels
 
 
 class Run(object):
     """`Run` stores the relevance scores estimated by the model under evaluation.<\br>
-    The preferred way for creating a `Run` istance is converting a Python dictionary as follows:
+    The preferred way for creating a `Run` instance is converting a Python dictionary as follows:
 
     ```python
     run_dict = {
@@ -42,9 +44,7 @@ class Run(object):
     ```
     """
 
-    def __init__(
-        self, run: Dict[str, Dict[str, float]] = None, name: str = None
-    ):
+    def __init__(self, run: Dict[str, Dict[str, float]] = None, name: str = None):
         if run is None:
             self.run = TypedDict.empty(
                 key_type=types.unicode_type,
@@ -72,6 +72,7 @@ class Run(object):
         self.metadata = {}
         self.scores = defaultdict(dict)
         self.mean_scores = {}
+        self.std_scores = {}
 
     def keys(self):
         """Returns query ids. Used internally."""
@@ -138,9 +139,28 @@ class Run(object):
         self.run = sort_dict_of_dict_by_value(self.run)
         self.sorted = True
 
+    def make_comparable(self, qrels: Qrels):
+        """Adds empty results for queries missing from the run and removes those not appearing in qrels."""
+        # Adds empty results for missing queries
+        for q_id in qrels.qrels:
+            if q_id not in self.run:
+                self.run[q_id] = create_empty_results_dict()
+
+        # Remove results for additional queries
+        to_remove = []
+        for q_id in self.run:
+            if q_id not in qrels.qrels:
+                to_remove.append(q_id)
+        for q_id in to_remove:
+            del self.run[q_id]
+
+        self.sort()
+
+        return self
+
     def to_typed_list(self):
         """Convert Run to Numba Typed List. Used internally."""
-        if self.sorted == False:
+        if not self.sorted:
             self.sort()
         return to_typed_list(self.run)
 
@@ -155,33 +175,48 @@ class Run(object):
             d[q_id] = dict(self[q_id])
         return d
 
+    def to_dataframe(self) -> pd.DataFrame:
+        """Convert Run to Pandas DataFrame with the following columns: `q_id`, `doc_id`, and `score`.
+
+        Returns:
+            pandas.DataFrame: Run as Pandas DataFrame.
+        """
+        data = {"q_id": [], "doc_id": [], "score": []}
+
+        for q_id in self.run:
+            for doc_id in self.run[q_id]:
+                data["q_id"].append(q_id)
+                data["doc_id"].append(doc_id)
+                data["score"].append(self.run[q_id][doc_id])
+
+        return pd.DataFrame.from_dict(data)
+
     def save(self, path: str = "run.json", kind: str = None):
-        """Write `run` to `path` as JSON file or TREC run format. File type is automatically inferred form the filename extension: ".json" -> "json", ".trec" -> "trec", ".txt" -> "trec". Use the "kind" argument to override this behavior.
+        """Write `run` to `path` as JSON file, TREC run, LZ4 file, or Parquet file. File type is automatically inferred form the filename extension: ".json" -> "json", ".trec" -> "trec", ".txt" -> "trec", and ".lz4" -> "lz4", ".parq" -> "parquet", ".parquet" -> "parquet". Use the "kind" argument to override this behavior.
 
         Args:
             path (str, optional): Saving path. Defaults to "run.json".
-            kind (str, optional): Kind of file to save, must be either "json" or "trec". If None, it will be automatically inferred from the filename extension.
+            kind (str, optional): Kind of file to save, must be either "json", "trec", or "ranxhub". If None, it will be automatically inferred from the filename extension.
         """
         # Infer file extension -------------------------------------------------
         kind = get_file_kind(path, kind)
 
         # Save Run -------------------------------------------------------------
-        if self.sorted == False:
+        if not self.sorted:
             self.sort()
 
         if kind == "json":
-            with open(path, "wb") as f:
-                f.write(
-                    orjson.dumps(self.to_dict(), option=orjson.OPT_INDENT_2)
-                )
+            save_json(self.to_dict(), path)
+        elif kind == "lz4":
+            save_lz4(self.to_dict(), path)
+        elif kind == "parquet":
+            self.to_dataframe().to_parquet(path, index=False)
         else:
             with open(path, "w") as f:
                 for i, q_id in enumerate(self.run.keys()):
                     for rank, doc_id in enumerate(self.run[q_id].keys()):
                         score = self.run[q_id][doc_id]
-                        f.write(
-                            f"{q_id} Q0 {doc_id} {rank+1} {score} {self.name}"
-                        )
+                        f.write(f"{q_id} Q0 {doc_id} {rank+1} {score} {self.name}")
 
                         if (
                             i != len(self.run.keys()) - 1
@@ -190,11 +225,12 @@ class Run(object):
                             f.write("\n")
 
     @staticmethod
-    def from_dict(d: Dict[str, Dict[str, float]]):
+    def from_dict(d: Dict[str, Dict[str, float]], name: str = None):
         """Convert a Python dictionary in form of {q_id: {doc_id: score}} to ranx.Run.
 
         Args:
             d (Dict[str, Dict[str, int]]): Run as Python dictionary
+            name (str, optional): Run name. Defaults to None.
 
         Returns:
             Run: ranx.Run
@@ -217,16 +253,18 @@ class Run(object):
         run = Run()
         run.run = create_and_sort(q_ids, doc_ids, scores)
         run.sorted = True
+        run.name = name
 
         return run
 
     @staticmethod
-    def from_file(path: str, kind: str = None):
-        """Parse a run file into ranx.Run. Supported formats are JSON and TREC run format. Correct import behavior is inferred from the file extension: ".json" -> "json", ".trec" -> "trec", ".txt" -> "trec". Use the "kind" argument to override this behavior.
+    def from_file(path: str, kind: str = None, name: str = None):
+        """Parse a run file into ranx.Run. Supported formats are JSON, TREC run, gzipped TREC run, and LZ4. Correct import behavior is inferred from the file extension: ".json" -> "json", ".trec" -> "trec", ".txt" -> "trec", ".gz" -> "gzipped trec", ".lz4" -> "lz4". Use the "kind" argument to override this behavior.
 
         Args:
             path (str): File path.
             kind (str, optional): Kind of file to load, must be either "json" or "trec".
+            name (str, optional): Run name. Defaults to None.
 
         Returns:
             Run: ranx.Run
@@ -236,21 +274,19 @@ class Run(object):
 
         # Load Run -------------------------------------------------------------
         if kind == "json":
-            run = orjson.loads(open(path, "rb").read())
+            run = load_json(path)
+        elif kind == "lz4":
+            run = load_lz4(path)
         else:
             run = defaultdict(dict)
-            name = ""
-            with open(path) as f:
+            with gzip.open(path, "rt") if kind == "gz" else open(path) as f:
                 for line in f:
                     q_id, _, doc_id, _, rel, run_name = line.split()
                     run[q_id][doc_id] = float(rel)
-                    if not name:
+                    if name is None:
                         name = run_name
 
-        run = Run.from_dict(run)
-
-        if kind == "trec":
-            run.name = name
+        run = Run.from_dict(run, name)
 
         return run
 
@@ -260,6 +296,7 @@ class Run(object):
         q_id_col: str = "q_id",
         doc_id_col: str = "doc_id",
         score_col: str = "score",
+        name: str = None,
     ):
         """Convert a Pandas DataFrame to ranx.Run.
 
@@ -268,18 +305,19 @@ class Run(object):
             q_id_col (str, optional): Query IDs column. Defaults to "q_id".
             doc_id_col (str, optional): Document IDs column. Defaults to "doc_id".
             score_col (str, optional): Relevance scores column. Defaults to "score".
+            name (str, optional): Run name. Defaults to None.
 
         Returns:
             Run: ranx.Run
         """
         assert (
             df[q_id_col].dtype == "O"
-        ), "DataFrame scores column dtype must be `object` (string)"
+        ), "DataFrame Query IDs column dtype must be `object` (string)"
         assert (
             df[doc_id_col].dtype == "O"
-        ), "DataFrame scores column dtype must be `object` (string)"
+        ), "DataFrame Document IDs column dtype must be `object` (string)"
         assert (
-            df[score_col].dtype == float
+            df[score_col].dtype == np.float64
         ), "DataFrame scores column dtype must be `float`"
 
         run_py = (
@@ -288,7 +326,39 @@ class Run(object):
             .to_dict()
         )
 
-        return Run.from_dict(run_py)
+        return Run.from_dict(run_py, name)
+
+    @staticmethod
+    def from_parquet(
+        path: str,
+        q_id_col: str = "q_id",
+        doc_id_col: str = "doc_id",
+        score_col: str = "score",
+        pd_kwargs: Dict[str, Any] = None,
+        name: str = None,
+    ):
+        """Convert a Parquet file to ranx.Run.
+
+        Args:
+            path (str): File path.
+            q_id_col (str, optional): Query IDs column. Defaults to "q_id".
+            doc_id_col (str, optional): Document IDs column. Defaults to "doc_id".
+            score_col (str, optional): Relevance scores column. Defaults to "score".
+            pd_kwargs (Dict[str, Any], optional): Additional arguments to pass to `pandas.read_parquet` (see https://pandas.pydata.org/docs/reference/api/pandas.read_parquet.html). Defaults to None.
+            name (str, optional): Run name. Defaults to None.
+
+        Returns:
+            Run: ranx.Run
+        """
+        pd_kwargs = {} if pd_kwargs is None else pd_kwargs
+
+        return Run.from_df(
+            df=pd.read_parquet(path, *pd_kwargs),
+            q_id_col=q_id_col,
+            doc_id_col=doc_id_col,
+            score_col=score_col,
+            name=name,
+        )
 
     @staticmethod
     def from_ranxhub(id: str):
@@ -325,16 +395,20 @@ class Run(object):
         return self.run.__str__()
 
 
-def get_file_kind(path: str = "qrels.json", kind: str = None) -> str:
+def get_file_kind(path: str = "run.json", kind: str = None) -> str:
     # Infer file extension
     if kind is None:
         kind = os.path.splitext(path)[1][1:]
         kind = "trec" if kind == "txt" else kind
+        kind = "parquet" if kind == "parq" else kind
 
     # Sanity check
     assert kind in {
         "json",
         "trec",
-    }, "Error `kind` must be 'json' or 'trec'"
+        "lz4",
+        "gz",
+        "parquet",
+    }, "Error `kind` must be 'json', 'trec', 'lz4', 'gz', or 'parquet'"
 
     return kind
