@@ -3,6 +3,7 @@ from .evaluate import evaluate
 from .fuse import fuse
 from .normalize import normalize
 from .optimize_fusion import optimize_fusion
+from .plot import plot
 
 __all__ = [
     "evaluate",
@@ -10,4 +11,5 @@ __all__ = [
     "fuse",
     "normalize",
     "optimize_fusion",
+    "plot",
 ]
