@@ -7,6 +7,8 @@ from .common import (
     create_empty_results_dict,
     create_empty_results_dict_list,
     extract_scores,
+    safe_max,
+    to_unicode,
 )
 
 
@@ -15,12 +17,14 @@ from .common import (
 def _max_norm(results):
     """Apply `max norm` to a given results dictionary."""
     scores = extract_scores(results)
-    max_score = max(scores)
+    max_score = safe_max(scores)
+
     denominator = max(max_score, 1e-9)
 
     normalized_results = create_empty_results_dict()
 
     for doc_id in results.keys():
+        doc_id = to_unicode(doc_id)
         normalized_results[doc_id] = results[doc_id] / denominator
 
     return normalized_results
